'*****************************************************************************
'PADS PowerPCB VB Macro - tpsummary203.bas  v2.03  31-01-06  Ron Olinyk
'Copyright 2009 Mindlink Technologies
'Description: Generates a testpoint summary report for current design. Reports
'             on all selected nets with testpoints -std format.
'Usage: Load and run script. Report created in design file directory.
'       If no nets selected, all nets are reported on.
'
'Updated 12-08-09 to provide with and without TP reports
'Updated 04-10-10 -check for Non-TP design
'*****************************************************************************

Option Explicit

Dim ary_nettp() As String
Dim ncnt As Long
Dim tpviacnt As Long
Dim tppincnt As Long
Dim no_tp As Boolean

Sub Main
Dim fmtary As Variant
Dim fmtary2 As Variant
Dim titary As Variant
Dim titary2 As Variant
Dim fpath As String
Dim tprep As String
Dim utp As Integer
Dim ltp As Integer
Dim notpnet As Integer
Dim ffmt As Integer
Dim lnn As Integer
Dim cnt As Long
Dim nn As String
Dim ccnt As Long
Dim colnet As Object
Dim nsel As Boolean

    On Error GoTo PrcsErr
	fpath = ActiveDocument.path
	tprep = "TPSummary.rep"
    fmtary = Array(0, 0, 8, 32, 44, 56)
    fmtary2 = Array(0, 0, 24, 32, 44)
    titary = Array("0", "PIN", "NETNAME", "X", "Y", "SIDE")
    titary2 = Array("0", "NETNAME", "VIAS", "SMD_PINS", "TH_PINS")
    
    Set colnet = ActiveDocument.GetObjects(ppcbObjectTypeNet,,True)
    nsel = True
    If colnet.Count = 0 Then
      Set colnet = ActiveDocument.GetObjects(ppcbObjectTypeNet,,)
      nsel = False
    End If
	ncnt = colnet.Count
	If ncnt = 0 Then
	  MsgBox "No nets in this design or design not loaded. Exiting.", vbExclamation, "Design Error"
	  End
	End If
	
		
	Open fpath & "\" & tprep For Output As #1
    StatusBarText = "Processing testpoints..."
	TPNetChk colnet
	StatusBarText = "Generating report..."
	utp = UBound(ary_nettp, 2)
	ltp = LBound(ary_nettp, 2)
	
	If utp > 0 Then
	  notpnet = IIf(ltp = 0, ltp + 1, ltp) + utp
	End If
	
	If utp = 0 Then
	  notpnet = IIf(no_tp, 0, 1)
	End If
	
	Print #1, "PowerPCB Testpoint Summary " & Choose(ActiveDocument.unit, "", "(mils)", "(inches)", "(mm)" ) & " for " & ActiveDocument.Name & "  " & Now
	Print #1
	Print #1, "Total Testpoints: " & tpviacnt + tppincnt & "  (Vias: " & tpviacnt & "; Pins: " & tppincnt & ")"
	Print #1, "Total" & IIf(nsel, " Selected ", " ") & "Nets: " & ncnt
	Print #1, "Total Nets with Testpoints: " & notpnet 
    Print #1, "Total Nets without Testpoints: " & ncnt - notpnet
    Print #1, "Percent Coverage (Nets with TP): " & Round(((notpnet) / ncnt) * 100, 1) & "%"
	Print #1
	Print #1, "-- Nets with a testpoint --"
	Print #1
	
  If utp = 0 And no_tp Then
	Print #1, "**** NO Testpoints found in design ****"
	Print #1
  Else
	
	Print #1, titary(1);
	ffmt = 0
	For ccnt = 2 To 5
	  lnn = Len(titary(ccnt - 1))
	  ffmt = fmtary(ccnt) - fmtary(ccnt - 1) - lnn
	  Print #1, Space(ffmt) & titary(ccnt);
	Next ccnt
	
	Print #1
	
	For cnt = ltp To utp
	   Print #1, ary_nettp(1, cnt);
	   ffmt = fmtary(1)
	   For ccnt = 2 To 5
	     nn = ary_nettp(ccnt - 1, cnt)
	     lnn = Len(nn)
	     ffmt = IIf(lnn < fmtary(ccnt), fmtary(ccnt) - lnn, 2) - fmtary(ccnt-1) 
	     Print #1, Space(ffmt) & ary_nettp(ccnt, cnt);
	   Next ccnt
	   Print #1
	Next cnt
	
	Print #1, String(fmtary(4)+Len(titary(4)),45)
	Print #1
	
  End If
  
	StatusBarText = "Processing testpoints..."
	TPNetChk2 colnet
	StatusBarText = "Generating report..."
	utp = UBound(ary_nettp, 2)
	ltp = LBound(ary_nettp, 2)
	notpnet = IIf(ltp = 0, ltp + 1, ltp) + utp

	Print #1, "-- Nets without a testpoint --"
	Print #1
	
  If utp = 0 And no_tp Then
	Print #1, "**** All nets (selected) have testpoints ****"
	Print #1
  Else
	
	Print #1, titary2(1);
	ffmt = 0
	For ccnt = 2 To 4
	  lnn = Len(titary2(ccnt - 1))
	  ffmt = fmtary2(ccnt) - fmtary2(ccnt - 1) - lnn - 1
	  Print #1, Space(ffmt) & titary2(ccnt);
	Next ccnt
	
	Print #1
	
	For cnt = ltp To utp
	   Print #1, ary_nettp(1, cnt);
	   ffmt = fmtary2(1)
	   For ccnt = 2 To 4
	     nn = ary_nettp(ccnt - 1, cnt)
	     lnn = Len(nn)
	     ffmt = IIf(lnn < fmtary2(ccnt), fmtary2(ccnt) - lnn, 2) - fmtary2(ccnt-1) 
	     Print #1, Space(ffmt) & ary_nettp(ccnt, cnt);
	   Next ccnt
	   Print #1
	Next cnt
	
	Print #1, String(fmtary2(4)+Len(titary2(4)),45)
  End If
  
    Close #1
    StatusBarText = ""
    MsgBox "Testpoint data processed. Report saved to:" & vbCr & _
           fpath & "\" & tprep, vbInformation, "TP Report Completed"
    
Exit Sub
PrcsErr:
  MsgBox "Process error encountered: " & Error$, vbExclamation, "Data Process Error"
  Close #1
End Sub

Private Sub TPNetChk(colnet As Object)
Dim cnt As Long
Dim tpfound As Boolean
Dim smpin As Long
Dim thpin As Long
Dim xnet As Object
Dim xvia As Object
Dim xpin As Object
Dim pinnam As String
Dim posx As Double
Dim posy As Double
Dim side As String

ReDim ary_nettp(1 To 5, 0 To 10000)

For Each xnet In colnet    
      tpfound = False
      smpin = 0
      thpin = 0
      For Each xvia In xnet.Vias
         If xvia.TestPoint <> ppcbTestPointNone Then
            tpfound = True
            tpviacnt = tpviacnt + 1
            pinnam = "Via"
            posx = xvia.PositionX
            posy = xvia.PositionY
            side = IIf (xvia.TestPoint = ppcbTestPointTopLayer, "TOP", "BOTTOM")
         End If
      Next xvia
        For Each xpin In xnet.Pins
          If xpin.IsSMD Then
            smpin = smpin + 1
          Else
            thpin = thpin + 1
          End If
          
          If xpin.TestPoint <> ppcbTestPointNone Then
            tpfound = True
            tppincnt = tppincnt + 1
            pinnam = xpin.Name
            posx = xpin.PositionX
            posy = xpin.PositionY
            side = IIf(xpin.TestPoint = ppcbTestPointTopLayer, "TOP", "BOTTOM")
          End If
        Next xpin
      
      If tpfound Then
         ary_nettp(1, cnt) = pinnam
         ary_nettp(2, cnt) = xnet.Name
         ary_nettp(3, cnt) = CStr(Format(posx,"0.0##"))
         ary_nettp(4, cnt) = CStr(Format(posy,"0.0##"))
         ary_nettp(5, cnt) = side
         cnt = cnt + 1
      End If
   Next xnet
   
   If cnt > 0 Then
      ReDim Preserve ary_nettp(1 To 5, 0 To cnt - 1)
   Else
      ReDim Preserve ary_nettp(1 To 5, 0)
      no_tp = True
   End If
   
End Sub

Private Sub TPNetChk2(colnet As Object)
Dim cnt As Long
Dim tpfound As Boolean
Dim smpin As Long
Dim thpin As Long
Dim xnet As Object
Dim xvia As Object
Dim xpin As Object

ReDim ary_nettp(1 To 4, 0 To 10000)

For Each xnet In colnet     
      tpfound = False
      smpin = 0
      thpin = 0
      For Each xvia In xnet.Vias
         If xvia.TestPoint <> ppcbTestPointNone Then
            tpfound = True
            'Exit For
            tpviacnt = tpviacnt + 1
         End If
      Next xvia
      'If Not tpfound Then
        For Each xpin In xnet.Pins
          If xpin.IsSMD Then
            smpin = smpin + 1
          Else
            thpin = thpin + 1
          End If
          
          If xpin.TestPoint <> ppcbTestPointNone Then
            tpfound = True
            tppincnt = tppincnt + 1
            'Exit For
          End If
        Next xpin
      'End If
      
      If Not tpfound Then
         ary_nettp(1, cnt) = xnet.Name
         ary_nettp(2, cnt) = xnet.Vias.Count
         ary_nettp(3, cnt) = CStr(smpin)
         ary_nettp(4, cnt) = CStr(thpin)
         cnt = cnt + 1
      End If
   Next xnet

   'ReDim Preserve ary_nettp(1 To 4, 0 To cnt - 1)
   If cnt > 0 Then
      ReDim Preserve ary_nettp(1 To 4, 0 To cnt - 1)
      no_tp = False
   Else
      ReDim Preserve ary_nettp(1 To 4, 0)
      no_tp = True
   End If
   
End Sub
